module jp.ku6.bbs.dao.ThreadDao;

import std.datetime;
import jp.ku6.firebird.core;
import jp.ku6.bbs.model.Thread;

/++
 + キーワードDAO
 +/
class ThreadDao
{
	/++
	 + コンストラクタ
	 +
	 + Params:
	 +  transaction = SQL実行に使用するトランザクションオブジェクト
	 +/
	this(Transaction transaction)
	{
		selectForListView_ = new Statement("SELECT t.id, t.title, t.registered, COUNT(t.id), MAX(r.registered) FROM responses AS r LEFT JOIN threads AS t ON r.thread_id = t.id GROUP BY t.id, t.title, t.registered", transaction);
	}

	/++
	 + キーワード毎の記事件数を取得
	 +
	 + Returns:
	 +  キーワードIDがキー、記事件数が値の連想配列
	 +/
	Thread[] selectForListView()
	{
		Statement statement = selectForListView_;

		// SQL実行の準備と終了処理
		statement.prepare();
		scope(exit) statement.finish();

		// SQL実行
		auto result = statement.execute();

		// データ取得
		Thread[] threads;
		while (result.fetch())
		{
			auto thread = new Thread;

			// データベースから取得したデータを型指定して取り出す
			thread.id = result.get!(uint)(0);
			thread.title = result.get!(string)(1);
			thread.registered = result.get!(SysTime)(2);
			thread.responseCount = result.get!(uint)(3);
			thread.updated = result.get!(SysTime)(4);

			threads ~= thread;
		}

		return threads;
	}

private:

	// 各SQLオブジェクト
	Statement selectForListView_;
}
